"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTgz = exports.writeTgz = exports.unGzip = exports.gzip = exports.downloadAndExtractFile = exports.npmInstallFlags = exports.isDirectory = exports.makeHttpRequest = exports.Fetcher = exports.streamDone = exports.stringOfStream = exports.streamOfString = exports.writeJson = exports.writeFile = exports.tryReadJson = exports.readJson = exports.readJsonSync = exports.readFileAndWarn = exports.readFileSync = exports.readFile = void 0;
require("./types/fstream");
const fs_extra_1 = require("fs-extra");
const tar_1 = require("tar");
const tar_stream_1 = __importDefault(require("tar-stream"));
const https_1 = __importStar(require("https"));
const path_1 = require("path");
const zlib_1 = __importDefault(require("zlib"));
const http_1 = require("http");
const stream_1 = require("stream");
const string_decoder_1 = require("string_decoder");
const miscellany_1 = require("./miscellany");
const fs_1 = require("./fs");
const assertions_1 = require("./assertions");
async function readFile(path) {
    const res = await (0, fs_extra_1.readFile)(path, { encoding: "utf8" });
    if (res.includes("�")) {
        throw new Error(`Bad character in ${path}`);
    }
    return res;
}
exports.readFile = readFile;
function readFileSync(path) {
    const res = (0, fs_extra_1.readFileSync)(path, { encoding: "utf8" });
    if (res.includes("�")) {
        throw new Error(`Bad character in ${path}`);
    }
    return res;
}
exports.readFileSync = readFileSync;
/** If a file doesn't exist, warn and tell the step it should have been generated by. */
async function readFileAndWarn(generatedBy, filePath) {
    try {
        return await readJson(filePath, miscellany_1.isObject);
    }
    catch (e) {
        console.error(`Run ${generatedBy} first!`);
        throw e;
    }
}
exports.readFileAndWarn = readFileAndWarn;
function readJsonSync(path, predicate) {
    return (0, miscellany_1.parseJson)(readFileSync(path), predicate);
}
exports.readJsonSync = readJsonSync;
async function readJson(path, predicate) {
    return (0, miscellany_1.parseJson)(await readFile(path), predicate);
}
exports.readJson = readJson;
async function tryReadJson(path, predicate) {
    return (0, miscellany_1.tryParseJson)(await readFile(path), predicate);
}
exports.tryReadJson = tryReadJson;
function writeFile(path, content) {
    return (0, fs_extra_1.writeFile)(path, content, { encoding: "utf8" });
}
exports.writeFile = writeFile;
function writeJson(path, content, formatted = true) {
    return (0, fs_extra_1.writeJson)(path, content, { spaces: formatted ? 4 : 0 });
}
exports.writeJson = writeJson;
function streamOfString(text) {
    const s = new stream_1.Readable();
    s.push(text);
    s.push(null); // tslint:disable-line no-null-keyword
    return s;
}
exports.streamOfString = streamOfString;
function stringOfStream(stream, description) {
    const decoder = new string_decoder_1.StringDecoder("utf8");
    let body = "";
    stream.on("data", (data) => {
        body += decoder.write(data);
    });
    return new Promise((resolve, reject) => {
        stream.on("error", reject);
        stream.on("end", () => {
            body += decoder.end();
            if (body.includes("�")) {
                reject(`Bad character decode in ${description}`);
            }
            else {
                resolve(body);
            }
        });
    });
}
exports.stringOfStream = stringOfStream;
function streamDone(stream) {
    return new Promise((resolve, reject) => {
        stream.on("error", reject).on("finish", resolve);
    });
}
exports.streamDone = streamDone;
class Fetcher {
    constructor() {
        this.agent = new https_1.Agent({ keepAlive: true });
    }
    async fetchJson(options) {
        const text = await this.fetch(options);
        try {
            return JSON.parse(text);
        }
        catch (e) {
            throw new Error(`Bad response from server:\noptions: ${JSON.stringify(options)}\n\n${text}`);
        }
    }
    async fetch(options) {
        const maxRetries = options.retries === false || options.retries === undefined ? 0 : options.retries === true ? 10 : options.retries;
        for (let retries = maxRetries; retries > 1; retries--) {
            try {
                return await doRequest(options, https_1.request, this.agent);
            }
            catch (err) {
                if (!/EAI_AGAIN|ETIMEDOUT|ECONNRESET/.test(err.message)) {
                    throw err;
                }
            }
            await (0, miscellany_1.sleep)(1);
        }
        return doRequest(options, https_1.request, this.agent);
    }
}
exports.Fetcher = Fetcher;
/** Only used for testing. */
function makeHttpRequest(options) {
    return doRequest(options, http_1.request);
}
exports.makeHttpRequest = makeHttpRequest;
function doRequest(options, makeRequest, agent) {
    return new Promise((resolve, reject) => {
        var _a;
        const req = makeRequest({
            hostname: options.hostname,
            port: options.port,
            path: `/${options.path}`,
            agent,
            method: options.method || "GET",
            headers: options.headers,
            timeout: (_a = options.timeout) !== null && _a !== void 0 ? _a : downloadTimeout,
        }, (res) => {
            let text = "";
            res.on("data", (d) => {
                text += d;
            });
            res.on("error", reject);
            res.on("end", () => {
                resolve(text);
            });
        });
        if (options.body !== undefined) {
            req.write(options.body);
        }
        req.end();
    });
}
async function isDirectory(path) {
    return (await (0, fs_extra_1.stat)(path)).isDirectory();
}
exports.isDirectory = isDirectory;
exports.npmInstallFlags = "--ignore-scripts --no-shrinkwrap --no-package-lock --no-bin-links --no-save --no-audit --no-fund --legacy-peer-deps";
const downloadTimeout = 1000000; // ms
const connectionTimeout = 800000; // ms
function downloadAndExtractFile(url, log) {
    return new Promise((resolve, reject) => {
        const timeout = setTimeout(reject, downloadTimeout);
        function rejectAndClearTimeout(reason) {
            clearTimeout(timeout);
            return reject(reason);
        }
        const root = new fs_1.Dir(undefined);
        function insertFile(path, content) {
            const components = path.split("/");
            const baseName = (0, assertions_1.assertDefined)(components.pop());
            let dir = root;
            for (const component of components) {
                dir = dir.subdir(component);
            }
            dir.set(baseName, content);
        }
        log.info("Requesting " + url);
        https_1.default
            .get(url, { timeout: connectionTimeout }, (response) => {
            if (response.statusCode !== 200) {
                return rejectAndClearTimeout(new Error(`DefinitelyTyped download failed with status code ${response.statusCode}`));
            }
            log.info("Getting " + url);
            const extract = tar_stream_1.default.extract();
            extract.on("entry", (header, stream, next) => {
                const name = (0, assertions_1.assertDefined)((0, miscellany_1.withoutStart)(header.name, "DefinitelyTyped-master/"));
                switch (header.type) {
                    case "file":
                        stringOfStream(stream, name)
                            .then((s) => {
                            insertFile(name, s);
                            next();
                        })
                            .catch(rejectAndClearTimeout);
                        break;
                    case "directory":
                        next();
                        break;
                    default:
                        throw new Error(`Unexpected file system entry kind ${header.type}`);
                }
            });
            extract.on("error", rejectAndClearTimeout);
            extract.on("finish", () => {
                log.info("Done receiving " + url);
                clearTimeout(timeout);
                resolve(new fs_1.InMemoryFS(root.finish(), "/"));
            });
            response.pipe(zlib_1.default.createGunzip()).pipe(extract);
        })
            .on("error", rejectAndClearTimeout);
    });
}
exports.downloadAndExtractFile = downloadAndExtractFile;
function gzip(input) {
    return input.pipe(zlib_1.default.createGzip());
}
exports.gzip = gzip;
function unGzip(input) {
    const output = zlib_1.default.createGunzip();
    input.pipe(output);
    return output;
}
exports.unGzip = unGzip;
function writeTgz(inputDirectory, outFileName) {
    return new Promise((resolve, reject) => {
        resolve(streamDone(createTgz(inputDirectory, reject).pipe((0, fs_extra_1.createWriteStream)(outFileName))));
    });
}
exports.writeTgz = writeTgz;
// To output this for testing:
// `require("./dist/io").createTgz("./src", err => { throw err }).pipe(fs.createWriteStream("foo.tgz"))`
function createTgz(dir, onError) {
    return gzip(createTar(dir, onError));
}
exports.createTgz = createTgz;
function createTar(dir, onError) {
    const dirSegments = (0, path_1.resolve)(dir).split(path_1.sep);
    const parentDir = dirSegments.slice(0, dirSegments.length - 1).join(path_1.sep);
    const entryToAdd = dirSegments[dirSegments.length - 1];
    const packer = new tar_1.Pack({ cwd: parentDir, filter: addDirectoryExecutablePermission });
    packer.on("error", onError);
    const stream = packer.add(entryToAdd);
    packer.end();
    return stream;
}
/**
 * Work around a bug where directories bundled on Windows do not have executable permission when extracted on Linux.
 * https://github.com/npm/node-tar/issues/7#issuecomment-17572926
 */
function addDirectoryExecutablePermission(_, stat) {
    if (stat.isDirectory()) {
        stat.mode = addExecutePermissionsFromReadPermissions(stat.mode);
    }
    return true;
}
function addExecutePermissionsFromReadPermissions(mode) {
    // Constant that gives execute permissions to owner, group, and others. "+x"
    const allExecutePermissions = 0o111;
    // Moves the bits for read permissions into the place for execute permissions.
    // In other words, a component will have execute permissions if it has read permissions.
    const readPermissionsAsExecutePermissions = (mode >>> 2) & allExecutePermissions; // tslint:disable-line no-bitwise
    // Add these additional execute permissions to the mode.
    return mode | readPermissionsAsExecutePermissions; // tslint:disable-line no-bitwise
}
//# sourceMappingURL=io.js.map